package org.opdyn;

class HegselmannKrause extends Model {

    double T;
    double[] alphas;
    double percentage;

    public HegselmannKrause(int agents, double lowerBound, double upperBound, double T, double alpha, double percentage) {
        
        super(agents, lowerBound, upperBound);
        this.T = T;
        this.alphas = new double[agents];

        int seekers = (int) (agents * percentage / 100.0);
        for (int i=0; i < seekers; i++) {
            this.alphas[i] = alpha;
        }
        for (int i=seekers; i < agents; i++) {
            this.alphas[i] = 0.0;
        }

    }

    @Override
    public void timeStep() {
        
        double[] newOpinions = new double[agents];
        double opinionSum, c;
        int confidenceSet;
    
        for (int i=0; i < agents; i++) {
            opinionSum = opinions[i];
            confidenceSet = 1;
            for (int j=0; j<agents; j++) {
                c = opinions[j] - opinions[i];
                if ((-lowerBound <= c) && (c <= upperBound)) {
                    if (i != j) {
                        opinionSum += opinions[j];
                        confidenceSet++;
                    }
            	}
            }
            
            newOpinions[i] = alphas[i] * T + (1.0 - alphas[i]) * opinionSum / ((double) confidenceSet);
            
        }
        
        opinions = newOpinions.clone();
        dataset.setNotify(false);
        for (int i=0; i < agents; i++) {
            dataset.getSeries(i).add(opinions[i], (double) (time+1));
        }
        dataset.setNotify(true);    

        compDistribution();    

        time++;

    }

}
