package org.opdyn;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;
import java.util.Hashtable;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Random;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.statistics.HistogramDataset;


public class Interface extends JApplet {
    
    int width, height;
    
    static final String TITLE = "Opinion Dynamics";
    
    Model model;
    int agents = 100;
    double lowerBound = 0.1;
    double upperBound = 0.1;
    double T = 0.0;
    double alpha = 0.0;
    double percentage = 0.0;
    
    // concerning slider to control update frequency
    static final int UPDATE_MIN = 1;
    static final int UPDATE_MAX = 100;
    static final int UPDATE_INIT = 5;
    
    // concerning slider to control number of agents
    static final int AGENTS_MIN = 10;
    static final int AGENTS_MAX = 1000;
    static final int AGENTS_INIT = 100;

    // concerning slider to control percentage of truth seekers
    static final int PERCENTAGE_MIN = 0;
    static final int PERCENTAGE_MAX = 100;
    static final int PERCENTAGE_INIT = 0;
    
    Timer timer;
    int delay = (int) 1000;  // timer delay for simulation step

    JRadioButton HK, DW;
    ChartPanel chartPanel, distPanel;
    JPanel panel, cardPanel;
    JButton pause, run, plotSwitch;
    CardLayout cardLayout;

    public void init() {

        // layout
        width = getSize().width;
        height = getSize().height;
        setSize(width, height);
        Font font = new Font("SansSerif", Font.PLAIN, 15);
              
        // start with some arbitrary model
        model = new HegselmannKrause(agents, lowerBound, upperBound, T, alpha, percentage);
        
        // create chart for this model
        JFreeChart chart = this.createChart();
        chartPanel = new ChartPanel(chart);  
        
        JFreeChart dist = this.createDistribution();
        distPanel = new ChartPanel(dist); 

        // run button
        run = new JButton("Start");
        run.setPreferredSize(new Dimension(100, 26));
        run.setFont(font);
                
        run.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String cmd = e.getActionCommand();
                if (cmd.equals("Start")) {
                    if (HK.isSelected()) {
                        model = new HegselmannKrause(agents, lowerBound, upperBound, T, alpha, percentage);
                    }
                    else {
                        model = new DeffuantWeisbuch(agents, lowerBound, upperBound);
                    }
                    
                    JFreeChart chart = Interface.this.createChart();
                    chartPanel.setChart(chart);
                    
                    JFreeChart dist = Interface.this.createDistribution();
                    distPanel.setChart(dist); 
                    
                    timer = new Timer(delay, new ActionListener() {
                        @Override
                        public void actionPerformed(ActionEvent e) {
                            model.timeStep();
                        }
                    });
                    timer.start();
                    run.setText("Stop"); pause.setText("Pause");
                } else {
                    timer.stop();
                    run.setText("Start");
                }
            }
        });
 
        // pause button
        pause = new JButton("Pause");
        pause.setPreferredSize(new Dimension(100, 26));
        pause.setFont(font);
                
        pause.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String cmd = e.getActionCommand();
                if (cmd.equals("Pause")) {
                    timer.stop();
                    pause.setText("Continue");
                } else {
                    timer.start();
                    pause.setText("Pause");
                }
            }
        });
        
        // agentSlider
        JSlider agentSlider = new JSlider(JSlider.HORIZONTAL, AGENTS_MIN, AGENTS_MAX, AGENTS_INIT);

        //Create the label table for agentSlider
        Hashtable<Integer, JLabel> agentTable = new Hashtable<Integer, JLabel>();
        agentTable.put(new Integer(AGENTS_MIN),
                       new JLabel(String.valueOf(AGENTS_MIN)) );
        agentTable.put(new Integer(AGENTS_MAX),
                       new JLabel(String.valueOf(AGENTS_MAX)) );
        agentSlider.setLabelTable(agentTable);

        agentSlider.setPaintTicks(true);
        agentSlider.setPaintLabels(true);
        agentSlider.setBorder(BorderFactory.createTitledBorder("Number of agents"));
        agentSlider.setToolTipText("Number of interacting agents");
        agentSlider.setBackground(Color.white);
        agentSlider.setFont(font);
    
        agentSlider.addChangeListener(new ChangeListener() { 
            @Override
            public void stateChanged(ChangeEvent e) {
            	JSlider source = (JSlider)e.getSource();
            	agents = source.getValue();
            }
        });

        // updateSlider
        JSlider updateSlider = new JSlider(JSlider.VERTICAL, UPDATE_MIN, UPDATE_MAX, UPDATE_INIT);

        //Create the label table for updateSlider.
        Hashtable<Integer, JLabel> updateTable = new Hashtable<Integer, JLabel>();
        updateTable.put(new Integer(UPDATE_MIN),
                       new JLabel("Slow") );
        updateTable.put(new Integer(UPDATE_MAX),
                       new JLabel("Fast") );
        updateSlider.setLabelTable(updateTable);

        updateSlider.setPaintTicks(true);
        updateSlider.setPaintLabels(true);
        updateSlider.setBorder(BorderFactory.createTitledBorder("Speed"));
        updateSlider.setToolTipText("Changes the speed of the simulation.");
        updateSlider.setBackground(Color.white);
        updateSlider.setFont(font);
        
        updateSlider.addChangeListener(new ChangeListener() { 
            @Override
            public void stateChanged(ChangeEvent e) {
            	JSlider source = (JSlider)e.getSource();
            	delay = (int) (5000.0 / ((double) source.getValue()));
                timer.setDelay(delay);
            }
        });


        // percentageSlider
        JSlider percentageSlider = new JSlider(JSlider.HORIZONTAL, PERCENTAGE_MIN, PERCENTAGE_MAX, PERCENTAGE_INIT);
        percentageSlider.setMajorTickSpacing(50);
        percentageSlider.setMinorTickSpacing(10);
        percentageSlider.setPaintTicks(true);
        percentageSlider.setPaintLabels(true);
        percentageSlider.setBorder(BorderFactory.createTitledBorder("Truth seekers (in %)"));
        percentageSlider.setBackground(Color.white);
        percentageSlider.setFont(font);
        
        percentageSlider.addChangeListener(new ChangeListener() { 
            @Override
            public void stateChanged(ChangeEvent e) {
            	JSlider source = (JSlider)e.getSource();
            	percentage = (double) source.getValue();
            }
        });

        
        // lowerBoundField
        JTextField lowerBoundField = new JTextField(String.valueOf(lowerBound), 8);
        lowerBoundField.setBorder(BorderFactory.createTitledBorder("Left bound"));
        lowerBoundField.setFont(font);
        
        lowerBoundField.addCaretListener(new CaretListener() { 
            @Override
            public void caretUpdate(CaretEvent e) {
            	JTextField source = (JTextField)e.getSource();
            	lowerBound = Double.parseDouble(source.getText());
            }
        });
        
        // upperBoundField
        JTextField upperBoundField = new JTextField(String.valueOf(upperBound), 8);
        upperBoundField.setBorder(BorderFactory.createTitledBorder("Right bound"));
        upperBoundField.setFont(font);
        
        upperBoundField.addCaretListener(new CaretListener() { 
            @Override
            public void caretUpdate(CaretEvent e) {
            	JTextField source = (JTextField)e.getSource();
            	upperBound = Double.parseDouble(source.getText());
            }
        });


        // trueValueField
        JTextField trueValueField = new JTextField(String.valueOf(T), 8);
        trueValueField.setBorder(BorderFactory.createTitledBorder("T"));
        trueValueField.setFont(font);
        
        trueValueField.addCaretListener(new CaretListener() { 
            @Override
            public void caretUpdate(CaretEvent e) {
            	JTextField source = (JTextField)e.getSource();
            	T = Double.parseDouble(source.getText());

            }
        });

        // strengthField
        JTextField strengthField = new JTextField(String.valueOf(alpha), 8);
        strengthField.setBorder(BorderFactory.createTitledBorder("alpha"));
        strengthField.setFont(font);
        
        strengthField.addCaretListener(new CaretListener() { 
            @Override
            public void caretUpdate(CaretEvent e) {
            	JTextField source = (JTextField)e.getSource();
            	alpha = Double.parseDouble(source.getText());

            }
        });

        
        // button to switch between plots
        plotSwitch = new JButton("Show distribution");
        plotSwitch.setPreferredSize(new Dimension(150, 26));
        plotSwitch.setFont(font);

        plotSwitch.addActionListener(new ActionListener() { 
            @Override
            public void actionPerformed(ActionEvent e) {
            	cardLayout.next(cardPanel);

                String cmd = e.getActionCommand();
                if (cmd.equals("Show distribution")) {
                    plotSwitch.setText("Show time chart");
                }
                else {
                    plotSwitch.setText("Show distribution");
                }

            }
        });
        
        
        // radio buttons to switch between models
        HK = new JRadioButton("HK");
        HK.setBackground(Color.white);
        HK.setSelected(true);
        DW = new JRadioButton("DW");
        DW.setBackground(Color.white);

        //Group the radio buttons.
        ButtonGroup group = new ButtonGroup();
        group.add(HK);
        group.add(DW);

        Box radioButtonBox = Box.createVerticalBox();
        radioButtonBox.setBackground(Color.white);
        radioButtonBox.setBorder(BorderFactory.createTitledBorder("Model"));
        radioButtonBox.setFont(font);
        radioButtonBox.add(HK);
        radioButtonBox.add(DW);

        
        // right box
        Box box = Box.createVerticalBox();
        box.setBackground(Color.white);
        box.add(updateSlider);
        box.add(lowerBoundField);
        box.add(upperBoundField);
        box.add(trueValueField);
        box.add(strengthField);
        
        
        JPanel boxPanel = new JPanel(new FlowLayout());
        boxPanel.setBackground(Color.white);
        boxPanel.add(box);
        

        // lower panel
        panel = new JPanel(new FlowLayout());
        panel.setBackground(Color.white);
        panel.add(run);
        panel.add(pause);
        panel.add(plotSwitch);
        panel.add(radioButtonBox);
        panel.add(agentSlider);
        panel.add(percentageSlider);
        
        
        // create panel containing both plots
        cardPanel = new JPanel(new CardLayout());
        cardPanel.setBackground(Color.white);

        cardPanel.add(chartPanel, "Card 1");
        cardPanel.add(distPanel, "Card 2");
        
        cardLayout = (CardLayout) cardPanel.getLayout();
        cardLayout.show(cardPanel, "Card 1");

        
        // put everything in content pane
        JPanel content = new JPanel();
        content.setBackground(Color.white);
        content.add(cardPanel); 
        content.add(cardPanel, BorderLayout.CENTER);
        content.add(boxPanel, BorderLayout.EAST);
        content.add(panel, BorderLayout.SOUTH);
        
        setContentPane(content);

        
    }
  
    
    public JFreeChart createChart() {

        JFreeChart chart = ChartFactory.createXYLineChart("Opinion Trajectories", 
                                               "opinion", 
                                               "time", 
                                               model.dataset, 
                                               PlotOrientation.HORIZONTAL,
                                               false,   // legend
                                               false,   // tooltips
                                               false);  // urls
        
        XYPlot plot = chart.getXYPlot();
        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer(true, false);

        Integer[] indices = new Integer[model.agents];
        for (int i=0; i<model.agents; i++) {
            indices[i] = i;
        }

        Arrays.sort(indices, model);
        
        int steps = (int) (model.agents / 4);
        int r = (int) (model.agents % 4);
        int[] allSteps = {steps, steps, steps, steps};
        for (int i=0; i < r; i++) {
            allSteps[i]++;
        }
        
        double stepSize;
        double[] colors = {1.0, 0.0, 0.0};
        int totalSteps = 0;
            
        for (int i=0; i<4; i++) {
            
            stepSize = 1.0 / ((double) allSteps[i]);
            
            for (int j=0; j<allSteps[i]; j++) {
                
                for (int c=0; c<3; c++) {
                    if (colors[c] < 0.0) {colors[c] = 0.0;}
                    if (colors[c] > 1.0) {colors[c] = 1.0;}   
                }
                
                Color color = new Color((float) colors[0], (float) colors[1], (float) colors[2]);
                renderer.setSeriesPaint(indices[totalSteps], color);
                
                if (i % 2 == 0) {
                    colors[i/2+1] += stepSize;  
                }
                else {     
                    colors[i/2] -= stepSize;  
                }
                
                totalSteps++;
                
            }
            
        }
        
        plot.setRenderer(renderer);

        ValueAxis domainAxis = plot.getDomainAxis();
        domainAxis.setRange(0.0, 1.0);
       
        ValueAxis rangeAxis = plot.getRangeAxis();
        rangeAxis.setAutoRange(true);  // todo: implement "running" axis
        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
        
        return chart;
        
    }
    
    public JFreeChart createDistribution() {
        
        JFreeChart distributionChart = ChartFactory.createXYLineChart("Opinion distribution", 
                                               "density", 
                                               "opinion", 
                                               model.distribution, 
                                               PlotOrientation.HORIZONTAL,
                                               false,   // legend
                                               false,   // tooltips
                                               false);  // urls
        
        XYPlot plot = distributionChart.getXYPlot();

        ValueAxis domainAxis = plot.getDomainAxis();
        domainAxis.setAutoRange(true);

        ValueAxis rangeAxis = plot.getRangeAxis();
        rangeAxis.setRange(0.0, 1.0);
      
        return distributionChart;

    }   

}
