package org.opdyn;

import java.util.Comparator;
import java.util.Random;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;


abstract class Model implements Comparator<Integer> {
    
    int agents;
    double lowerBound;
    double upperBound;
    XYSeriesCollection dataset = new XYSeriesCollection();
    XYSeriesCollection distribution = new XYSeriesCollection();
    int time = 0;
    double[] opinions;
    int bins;
    
    Model(int agents, double lowerBound, double upperBound) {
        
        this.agents = agents;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
        this.opinions = new double[agents];
        this.bins = agents;
        
        // setting random opinions at t=0
        Random rn = new Random();
        for(int i = 0; i < agents; i++) {
            XYSeries series = new XYSeries("Opinion of agent " + Integer.toString(i), false, true);  // no autosort, allow duplicates
            double op = rn.nextDouble();
            opinions[i] = op;
            series.add(op, 0.0);
            dataset.addSeries(series);
        }

        XYSeries series = new XYSeries("Distribution at time 0", false, true);
        distribution.addSeries(series);
    
    }

    public abstract void timeStep();

    public void compDistribution() {
        
        double[] dist = new double[bins];
        int bin;
        
        for(int i = 0; i< agents; i++){
            bin = (int) (opinions[i] * bins);
            dist[bin] += 1.0 / ((double) agents);
        }
       
        distribution.setNotify(false);
        distribution.removeSeries(0);
        
        XYSeries series = new XYSeries("Distribution at time " + (time+1), false, true);
        for (int i=0; i < bins; i++) {
            series.add((double) dist[i], ((double) i +0.5) / ((double) bins));
        }
        distribution.addSeries(series);
        
        distribution.setNotify(true);
        
    }
    
    // compares two agents by using their initial opinion
    @Override
    public int compare(Integer o1, Integer o2) {
        return Double.compare(opinions[o1], opinions[o2]);
    }
       
}
